@@ -31,7 +31,7 @@
 
 extern char last_error[];
 
-int connect_to(struct sockaddr *bind_to, struct addrinfo *ai, int timeout)
+int connect_to(struct sockaddr *bind_to, struct addrinfo *ai, int timeout, char *tfo, char *msg, int msg_len, int *msg_accepted)
 {
 	int     fd;
 	int 	rc;
@@ -81,24 +81,45 @@ int connect_to(struct sockaddr *bind_to, struct addrinfo *ai, int timeout)
 	to.tv_usec = (timeout - (to.tv_sec * 1000)) * 1000;
 
 	/* connect to peer */
-	if (connect(fd, ai -> ai_addr, ai -> ai_addrlen) == 0)
+#ifdef TCP_TFO
+	if (*tfo)
 	{
-		/* connection made, return */
-		return fd;
+		rc = sendto(fd, msg, msg_len, MSG_FASTOPEN, ai -> ai_addr, ai -> ai_addrlen);
+		
+		if(rc == msg_len)
+			*msg_accepted = 1;
+		if(errno == 0)
+			return fd;
+		if(errno == ENOTSUP)
+		{
+			printf("TCP TFO Not Supported. Please check if \"/proc/sys/net/ipv4/tcp_fastopen\" is 1. Disabling TFO for now.\n");
+			*tfo = 0;
+		}
+	}
+			
+	else
+#endif
+	{
+		if (connect(fd, ai -> ai_addr, ai -> ai_addrlen) == 0)
+		{
+			/* connection made, return */
+			return fd;
+		}
 	}
 
 	/* wait for connection */
 	rc = select(fd + 1, NULL, &wfds, NULL, &to);
 	if (rc == 0)
 	{
+		snprintf(last_error, ERROR_BUFFER_SIZE, "connect time out\n");
 		close (fd);
-		return -2;	/* timeout */
+		return RC_TIMEOUT;	/* timeout */
 	}
 	else if (rc == -1)
 	{
 		close(fd);
 		if (errno == EINTR)
-			return -3;	/* ^C pressed */
+			return RC_CTRLC;/* ^C pressed */
 		else
 			return -1;	/* error */
 	}
